RetentionPolicy

Struct RetentionPolicy 

pub struct RetentionPolicy {
    pub number_of_journal_files: Option<usize>,
    pub size_of_journal_files: Option<u64>,
    pub duration_of_journal_files: Option<Duration>,
}
Controls when old journal files should be deleted.

Old files are removed to satisfy all configured limits. Removal starts with the oldest files first. If all fields are None, files are never deleted.

Fields

number_of_journal_files: Option<usize>

Maximum number of journal files to keep

size_of_journal_files: Option<u64>

Maximum total size of all journal files (in bytes)

duration_of_journal_files: Option<Duration>

Maximum age of files to keep

Implementations

Source

impl RetentionPolicy

pub fn with_number_of_journal_files( self, number_of_journal_files: usize, ) -> RetentionPolicy

Specifies maximum number of journal files.

pub fn with_size_of_journal_files( self, size_of_journal_files: u64, ) -> RetentionPolicy

Specifies maximum size of journal files.

pub fn with_duration_of_journal_files( self, duration_of_journal_files: Duration, ) -> RetentionPolicy

Specifies maximum duration of journal files.