Crate journal

Crate journal 

Pure-Rust systemd journal reader and writer SDK.

This crate provides a public Rust layer over the imported Netdata journal reader/writer crates. It intentionally keeps the low-level file parsing in the imported implementation and adds byte-safe entries, directory reading, export/JSON formatting, and a libsystemd-style facade.

Modules

netdata

Structs

BootInfo
BucketUtilization
Hash table bucket utilization statistics
Config
Configuration for a journal log.
DirectoryReader
Entry
EntryTimestamps
ExplorerComparison
ExplorerControl
ExplorerFilter
ExplorerFtsPattern
ExplorerHistogram
ExplorerHistogramBucket
ExplorerProgress
ExplorerQuery
ExplorerResult
ExplorerRow
ExplorerSampling
ExplorerStats
Field
FileHeader
FileReader
JournalFile
A reader for systemd journal files that efficiently maps small regions of the file into memory.
JournalReader
Log
Tracks rotation state for size and count limits.
Mmap
A handle to an immutable memory mapped buffer.
Origin
Origin identifies where a journal file comes from
RawField
ReaderOptions
RetentionPolicy
Controls when old journal files should be deleted.
RotationPolicy
Controls when journal files should be rotated
SdJournal

Enums

Compression
Direction
EntryItemsType
ExplorerAnchor
ExplorerFieldMode
ExplorerStopReason
ExplorerStrategy
FacadeError
FieldNamePolicy
JournalError
Location
LogLifecycleEvent
OutputMode
ReaderBounds
SdkError
Source of journal entries
WriterError
Errors that can occur during journal writing operations.

Constants

DEFAULT_READER_WINDOW_SIZE
ERR_END_OF_ENTRIES
ERR_INVALID_CURSOR
ERR_NO_ENTRY
ERR_UNSUPPORTED

Traits

HashableObject
LogLifecycleObserver

Functions

SdJournalAddConjunction
SdJournalAddDisjunction
SdJournalAddMatch
SdJournalClose
SdJournalEnumerateAvailableData
SdJournalEnumerateAvailableUnique
SdJournalEnumerateField
SdJournalEnumerateFields
SdJournalFlushMatches
SdJournalGetCursor
SdJournalGetData
SdJournalGetEntry
SdJournalGetMonotonicUsec
SdJournalGetRealtimeUsec
SdJournalGetSeqnum
SdJournalListBoots
SdJournalNext
SdJournalNextSkip
SdJournalOpen
SdJournalOpenDirectory
SdJournalOpenDirectoryWithOptions
SdJournalOpenFile
SdJournalOpenFileWithOptions
SdJournalOpenFiles
SdJournalOpenFilesWithOptions
SdJournalPrevious
SdJournalPreviousSkip
SdJournalProcessOutput
SdJournalQueryUnique
SdJournalQueryUniqueState
SdJournalRestartData
SdJournalRestartFields
SdJournalRestartUnique
SdJournalSeekCursor
SdJournalSeekHead
SdJournalSeekRealtimeUsec
SdJournalSeekTail
SdJournalSetOutputMode
SdJournalTestCursor
SdJournalVisitUniqueValues
export_entry
export_entry_bytes
format_entry_text
json_entry
parse_cursor
parse_match_bytes
parse_match_string
verify_file
Validate the structural integrity of a journal file.
verify_file_with_key
Validate the integrity of a journal file with a verification key.

Type Aliases

ParseError
ParsedCursor
Result